@@ -33,7 +33,8 @@
 #' @param DummyPrefix character vector of length 2 with prefixes for dummy
 #'   dams (mothers) and sires (fathers); maximum 20 characters each.
 #' @param Complexity Either "full" (default), "simp" (no explicit consideration
-#'   of inbred relationships) or "mono" (monogamous breeding system)
+#'   of inbred relationships), "mono" (monogamous breeding system), or "herm"
+#'   (hermaphrodites)
 #' @param FindMaybeRel  Identify pairs of non-assigned likely relatives after
 #'   pedigree reconstruction. Can be time-consuming in large datasets.
 #' @param CalcLLR  Calculate log-likelihood ratios for all assignments. Can be
@@ -58,12 +59,9 @@ SeqPrep <- function(GenoM = NULL,
             				 CalcLLR = TRUE)
 
 {
-  if (!is.matrix(GenoM)) stop("'GenoM' should be a numeric matrix")
-  if (!all(GenoM %in% c(0,1,2,-9))) stop("'GenoM' not in <seq> format, please use GenoConvert")
   nIndG <- nrow(GenoM)
   IDs_geno <- rownames(GenoM)
   nSnp <- ncol(GenoM)
-  if (any(duplicated(IDs_geno)))  stop("'GenoM' has duplicate IDs. Please exclude or rename these samples, or run GenoConvert with UseFID=TRUE.")
 
   if (Err<0 || Err>1 || !is.double(Err)) stop("invalid value for 'Err'")
   if (MaxMismatch<0 || !is.wholenumber(MaxMismatch))  stop("invalid value for 'MaxMismatch'")
@@ -72,7 +70,7 @@ SeqPrep <- function(GenoM = NULL,
   if (MaxSibshipSize<0 || !is.wholenumber(MaxSibshipSize)) {
     stop("invalid value for MaxSibshipSize")
   }
-  if (!Complexity %in% c("full", "simp", "mono")) stop("invalid value for 'Complexity'")
+  if (!Complexity %in% c("full", "simp", "mono", "herm")) stop("invalid value for 'Complexity'")
   if (!UseAge %in% c("yes", "no", "extra")) stop("invalid value for 'UseAge'")
   if (!FindMaybeRel %in% c(TRUE, FALSE)) stop("invalid value for 'FindMaybeRel'")
   if (!CalcLLR %in% c(TRUE, FALSE)) stop("invalid value for 'CalcLLR'")
@@ -109,3 +107,45 @@ SeqPrep <- function(GenoM = NULL,
   Specs
 }
 
+
+
+#=======================================================================
+#' @title check GenoM
+#'
+#' @description Check that the provided genotype matrix is in the correct format
+#'
+#' @param GenoM the genotype matrix
+#'
+#' @return updates GenoM internally to remove low call rate individuals (<.5)
+#'   and low call rate SNPs (<0.1), if necessary
+#'
+#' @keywords internal
+
+CheckGeno <- function(GenoM) {
+	if (is.null(GenoM)) stop("please provide 'GenoM'")
+  if (!is.matrix(GenoM)) stop("'GenoM' should be a numeric matrix")
+	if (!all(GenoM %in% c(0,1,2,-9))) stop("'GenoM' not in <seq> format, please use GenoConvert")
+
+  if (is.null(rownames(GenoM))) stop("'GenoM' has no rownames, these should be the individual IDs")
+	if (any(duplicated(rownames(GenoM))))  stop("'GenoM' has duplicate IDs. Please exclude or rename these samples, or run GenoConvert with UseFID=TRUE.")
+
+  Excl <- list()
+	Lscored <- apply(GenoM, 1, function(x) sum(x!=-9))
+  if (any(Lscored < ncol(GenoM)/2)) {
+    warning(paste("There are ", sum(Lscored < ncol(GenoM)/2)," individuals scored for <50% of SNPs, these will be excluded"),
+            immediate.=TRUE)
+		GenoM <<- GenoM[Lscored >= ncol(GenoM)/2, ]
+		Excl[["ExcludedInd"]] <- rownames(GenoM)[which(Lscored < ncol(GenoM)/2)]
+  }
+
+  Nscored <- apply(GenoM, 2, function(x) sum(x!=-9))
+  if (any(Nscored < nrow(GenoM)/10)) {
+    warning(paste("There are ", sum(Nscored < nrow(GenoM)/10)," SNPs scored for <10% of individuals, these will be excluded"),
+            immediate.=TRUE)
+		GenoM <<- GenoM[, Nscored >= ncol(GenoM)/10]
+		Excl[["ExcludedSnps"]] <- which(Nscored < nrow(GenoM)/10)
+  }
+  Excl
+}
+
+
